@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2008-2012 by Andrzej Rybczak                            *
+ *   Copyright (C) 2008-2014 by Andrzej Rybczak                            *
  *   electricityispower@gmail.com                                          *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -18,29 +18,30 @@
  *   51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.              *
  ***************************************************************************/
 
+#include "config.h"
 #include "curl_handle.h"
 
 #ifdef HAVE_CURL_CURL_H
 
 #include <cstdlib>
+#include <cstring>
+#include <boost/algorithm/string/replace.hpp>
+#include <boost/algorithm/string/trim.hpp>
+#include <boost/regex.hpp>
 
 #include "charset.h"
-#include "conv.h"
 #include "lyrics_fetcher.h"
+#include "utility/html.h"
+#include "utility/string.h"
 
 LyricsFetcher *lyricsPlugins[] =
 {
 	new LyricwikiFetcher(),
-	new LyricsvipFetcher(),
+	new AzLyricsFetcher(),
 	new Sing365Fetcher(),
-	new LoloLyricsFetcher(),
-	new LyriczzFetcher(),
-	new SonglyricsFetcher(),
 	new LyricsmaniaFetcher(),
-	new LyricstimeFetcher(),
 	new MetrolyricsFetcher(),
 	new JustSomeLyricsFetcher(),
-	new LyrcComArFetcher(),
 	new InternetLyricsFetcher(),
 	0
 };
@@ -52,9 +53,9 @@ LyricsFetcher::Result LyricsFetcher::fetch(const std::string &artist, const std:
 	Result result;
 	result.first = false;
 	
-	std::string url = getURL();
-	Replace(url, "%artist%", artist.c_str());
-	Replace(url, "%title%", title.c_str());
+	std::string url = urlTemplate();
+	boost::replace_all(url, "%artist%", artist);
+	boost::replace_all(url, "%title%", title);
 	
 	std::string data;
 	CURLcode code = Curl::perform(data, url);
@@ -65,41 +66,46 @@ LyricsFetcher::Result LyricsFetcher::fetch(const std::string &artist, const std:
 		return result;
 	}
 	
-	bool parse_ok = getContent(getOpenTag(), getCloseTag(), data);
+	auto lyrics = getContent(regex(), data);
 	
-	if (!parse_ok || notLyrics(data))
+	if (lyrics.empty() || notLyrics(data))
 	{
 		result.second = msgNotFound;
 		return result;
 	}
 	
-	postProcess(data);
+	data.clear();
+	for (auto it = lyrics.begin(); it != lyrics.end(); ++it)
+	{
+		postProcess(*it);
+		if (!it->empty())
+		{
+			data += *it;
+			if (it != lyrics.end()-1)
+				data += "\n\n----------\n\n";
+		}
+	}
 	
 	result.second = data;
 	result.first = true;
 	return result;
 }
 
-bool LyricsFetcher::getContent(const char *open_tag, const char *close_tag, std::string &data)
+std::vector<std::string> LyricsFetcher::getContent(const char *regex_, const std::string &data)
 {
-	size_t a, b;
-	if ((a = data.find(open_tag)) != std::string::npos)
-	{
-		a += strlen(open_tag);
-		if ((b = data.find(close_tag, a)) != std::string::npos)
-			data = data.substr(a, b-a);
-		else
-			return false;
-	}
-	else
-		return false;
-	return true;
+	std::vector<std::string> result;
+	boost::regex rx(regex_);
+	auto first = boost::sregex_iterator(data.begin(), data.end(), rx);
+	auto last = boost::sregex_iterator();
+	for (; first != last; ++first)
+		result.push_back(first->str(1));
+	return result;
 }
 
 void LyricsFetcher::postProcess(std::string &data)
 {
-	StripHtmlTags(data);
-	Trim(data);
+	stripHtmlTags(data);
+	boost::trim(data);
 }
 
 /***********************************************************************/
@@ -120,23 +126,36 @@ LyricsFetcher::Result LyricwikiFetcher::fetch(const std::string &artist, const s
 			return result;
 		}
 		
-		bool parse_ok = getContent("'17'/></a></div>", "<!--", data);
+		auto lyrics = getContent("<div class='lyricbox'><script>.*?</script>(.*?)<!--", data);
 		
-		if (!parse_ok)
+		if (lyrics.empty())
 		{
 			result.second = msgNotFound;
 			return result;
 		}
-		data = unescapeHtmlUtf8(data);
-		if (data.find("Unfortunately, we are not licensed to display the full lyrics for this song at the moment.") != std::string::npos)
+		std::transform(lyrics.begin(), lyrics.end(), lyrics.begin(), unescapeHtmlUtf8);
+		bool license_restriction = std::any_of(lyrics.begin(), lyrics.end(), [](const std::string &s) {
+			return s.find("Unfortunately, we are not licensed to display the full lyrics for this song at the moment.") != std::string::npos;
+		});
+		if (license_restriction)
 		{
 			result.second = "Licence restriction";
 			return result;
 		}
 		
-		Replace(data, "<br />", "\n");
-		StripHtmlTags(data);
-		Trim(data);
+		data.clear();
+		for (auto it = lyrics.begin(); it != lyrics.end(); ++it)
+		{
+			boost::replace_all(*it, "<br />", "\n");
+			stripHtmlTags(*it);
+			boost::trim(*it);
+			if (!it->empty())
+			{
+				data += *it;
+				if (it != lyrics.end()-1)
+					data += "\n\n----------\n\n";
+			}
+		}
 		
 		result.second = data;
 		result.first = true;
@@ -159,8 +178,8 @@ LyricsFetcher::Result GoogleLyricsFetcher::fetch(const std::string &artist, cons
 	std::string search_str = artist;
 	search_str += "+";
 	search_str += title;
-	search_str += "+";
-	search_str += getSiteKeyword();
+	search_str += "+%2B";
+	search_str += siteKeyword();
 	
 	std::string google_url = "http://www.google.com/search?hl=en&ie=UTF-8&oe=UTF-8&q=";
 	google_url += search_str;
@@ -175,17 +194,15 @@ LyricsFetcher::Result GoogleLyricsFetcher::fetch(const std::string &artist, cons
 		return result;
 	}
 	
-	bool found_url = getContent("<A HREF=\"", "\">here</A>", data);
+	auto urls = getContent("<A HREF=\"(.*?)\">here</A>", data);
 	
-	if (!found_url || !isURLOk(data))
+	if (urls.empty() || !isURLOk(urls[0]))
 	{
 		result.second = msgNotFound;
 		return result;
 	}
 	
-	data = unescapeHtmlUtf8(data);
-	//result.second = data;
-	//return result;
+	data = unescapeHtmlUtf8(urls[0]);
 	
 	URL = data.c_str();
 	return LyricsFetcher::fetch("", "");
@@ -193,24 +210,15 @@ LyricsFetcher::Result GoogleLyricsFetcher::fetch(const std::string &artist, cons
 
 bool GoogleLyricsFetcher::isURLOk(const std::string &url)
 {
-	return url.find(getSiteKeyword()) != std::string::npos;
+	return url.find(siteKeyword()) != std::string::npos;
 }
 
 /**********************************************************************/
 
-bool LyricstimeFetcher::isURLOk(const std::string &url)
+void Sing365Fetcher::postProcess(std::string &data)
 {
-	// it sometimes returns list of all artists that begin
-	// with a given letter, e.g. www.lyricstime.com/A.html, which
-	// is 25 chars long, so we want longer.
-	return GoogleLyricsFetcher::isURLOk(url) && url.length() > 25;
-}
-
-void LyricstimeFetcher::postProcess(std::string &data)
-{
-	// lyricstime.com uses iso-8859-1 as the encoding
-	// so we need to convert obtained lyrics to utf-8
-	iconv_convert_from_to("iso-8859-1", "utf-8", data);
+	// throw away ad
+	data = boost::regex_replace(data, boost::regex("<div.*</div>"), "");
 	LyricsFetcher::postProcess(data);
 }
 
@@ -218,15 +226,11 @@ void LyricstimeFetcher::postProcess(std::string &data)
 
 void MetrolyricsFetcher::postProcess(std::string &data)
 {
-	// throw away [ from ... ] info
-	size_t i = data.find('['), j = data.find(']');
-	if (i != std::string::npos && i != std::string::npos)
-		data.replace(i, j-i+1, "");
 	// some of lyrics have both \n chars and <br />, html tags
 	// are always present whereas \n chars are not, so we need to
 	// throw them away to avoid having line breaks doubled.
-	Replace(data, "&#10;", "");
-	Replace(data, "<br />", "\n");
+	boost::replace_all(data, "&#10;", "");
+	boost::replace_all(data, "<br />", "\n");
 	data = unescapeHtmlUtf8(data);
 	LyricsFetcher::postProcess(data);
 }
@@ -239,30 +243,6 @@ bool MetrolyricsFetcher::isURLOk(const std::string &url)
 
 /**********************************************************************/
 
-void LyricsmaniaFetcher::postProcess(std::string &data)
-{
-	// lyricsmania.com uses iso-8859-1 as the encoding
-	// so we need to convert obtained lyrics to utf-8
-	iconv_convert_from_to("iso-8859-1", "utf-8", data);
-	LyricsFetcher::postProcess(data);
-}
-
-/**********************************************************************/
-
-void SonglyricsFetcher::postProcess(std::string &data)
-{
-	// throw away [ ... lyrics are found on www.songlyrics.com ] info.
-	// there is +2 instead of +1 in third line because there is extra
-	// space after ] we also want to get rid of
-	size_t i = data.find('['), j = data.find(']');
-	if (i != std::string::npos && i != std::string::npos)
-		data.replace(i, j-i+2, "");
-	data = unescapeHtmlUtf8(data);
-	LyricsFetcher::postProcess(data);
-}
-
-/**********************************************************************/
-
 LyricsFetcher::Result InternetLyricsFetcher::fetch(const std::string &artist, const std::string &title)
 {
 	GoogleLyricsFetcher::fetch(artist, title);
